@@ -1,12 +1,16 @@
 # Created by matveyev at 26.01.2021
 
+import os
 import xml.etree.cElementTree as ET
+import PyTango
 
 from PyQt5 import QtWidgets, QtCore
 
 from onlinexml_editor.gui.new_device_ui import Ui_AddDevice
 from onlinexml_editor.property_widget import PropertyWidget
-from onlinexml_editor.devices_class import SerialDeviceNode, GroupNode, ConfigurationNode
+from onlinexml_editor.devices_class import SerialDeviceNode, GroupNode
+
+ALWAYS_PERSONAL = ['active', 'comment', 'name']
 
 
 class ConfigureDevice(QtWidgets.QDialog):
@@ -31,39 +35,59 @@ class ConfigureDevice(QtWidgets.QDialog):
         self._ui.fr_common_properties.setVisible(False)
         self._ui.fr_personal_properties.setVisible(False)
 
-        self._ui.cmb_type.currentTextChanged.connect(self._change_device_type)
+        self._ui.cmd_apply_template.clicked.connect(self._apply_template)
+        self._ui.cmb_template.currentIndexChanged.connect(self._apply_template)
+
         self._ui.cmd_add_common_property.clicked.connect(lambda: self._add_property('common'))
         self._ui.cmd_add_personal_property.clicked.connect(lambda: self._add_property('personal'))
 
-        if options['new']:
+        self._default_color = self._ui.le_name.styleSheet()
+        self._ui.le_name.textEdited.connect(self._colorize_ui)
 
+        self.templates = ET.parse(os.path.join(os.path.dirname(__file__), 'default_templates.xml')).getroot()
+
+        for device in self.templates.iter('device'):
+            if device.get('name') is not None:
+                self._ui.cmb_template.addItem(device.get('name'))
+
+        if options['new']:
             self._type = options['type']
             self.new_device = True
 
-            if self._type == 'non_serial':
-                self._ui.cmb_type.addItems(['group', 'serial_device', 'single_device'])
+            if self._type in ['group', 'configuration']:
+                self._ui.fr_template.setVisible(False)
 
-            elif self._type == 'configuration':
-                self._ui.cmb_type.addItems(['configuration'])
+            elif self._type == 'serial_device':
+                self._ui.fr_common_properties.setVisible(True)
+                self._ui.fr_personal_properties.setVisible(True)
 
             else:
-                self._ui.cmb_type.addItem('single_device')
+                if type(options['parent']) == SerialDeviceNode:
+                    self._ui.fr_template.setVisible(False)
+                    self._ui.fr_common_properties.setVisible(True)
+                    personal, common = self._parse_serial_device(options['parent'])
+                    for key in personal:
+                        self._add_property('personal', key, '', doRefresh=False)
+                    for key, value in common.items():
+                        self._add_property('common', key, value, editable=False, doRefresh=False)
+
                 self._ui.fr_personal_properties.setVisible(True)
-                self._add_property('personal', 'device', '', True)
+
+            self._ui.le_name.setStyleSheet("QLineEdit {background: rgb(255, 102, 74);}")
         else:
-            self._ui.cmb_type.addItem(options['device'].class_type())
-            self._ui.cmb_type.setEnabled(False)
+            self._ui.fr_template.setVisible(False)
 
             self.new_device = False
             self.edited_device = options['device']
             self._sub_device = options['sub_device']
 
-            if isinstance(self.edited_device, SerialDeviceNode):
+            if type(self.edited_device) == SerialDeviceNode:
                 self._type = 'serial'
+
+                personal, common = self._parse_serial_device(self.edited_device)
                 self._ui.fr_common_properties.setVisible(True)
-                for key, value in self.edited_device.info.items():
-                    if key not in ['active', 'comment', 'name']:
-                        self._add_property('common', key, value, False)
+                for key, value in common.items():
+                    self._add_property('common', key, value, doRefresh=False)
 
                 device = self.edited_device.children[self._sub_device]
                 self._ui.fr_personal_properties.setVisible(True)
@@ -71,19 +95,45 @@ class ConfigureDevice(QtWidgets.QDialog):
                 self._ui.le_comment.setText(device.info['comment'])
 
                 for key, value in device.info.items():
-                    if key not in ['active', 'comment', 'name']:
-                        self._add_property('personal', key, value, False)
+                    if key not in ALWAYS_PERSONAL:
+                        self._add_property('personal', key, value, doRefresh=False)
 
             else:
-                self._type = 'non_serial'
+                self._type = 'device'
                 self._ui.fr_personal_properties.setVisible(True)
                 self._ui.le_name.setText(self.edited_device.info['name'])
                 self._ui.le_comment.setText(self.edited_device.info['comment'])
                 for key, value in self.edited_device.info.items():
-                    if key not in ['active', 'comment', 'name']:
-                        self._add_property('personal', key, value, False)
+                    if key not in ALWAYS_PERSONAL:
+                        self._add_property('personal', key, value, doRefresh=False)
 
-            self._bild_view()
+        self._bild_view()
+
+    # ----------------------------------------------------------------------
+    def _parse_serial_device(self, device):
+
+        common = dict(device.info)
+        for key in ALWAYS_PERSONAL:
+            if key in common:
+                del common[key]
+
+        all_keys = []
+        for child in device.children:
+            all_keys += list(child.info.keys())
+
+        all_keys = list(set(all_keys))
+        for key in ALWAYS_PERSONAL:
+            if key in all_keys:
+                all_keys.remove(key)
+
+        return all_keys, common
+
+    # ----------------------------------------------------------------------
+    def _colorize_ui(self):
+        if self._ui.le_name.text() == "":
+            self._ui.le_name.setStyleSheet("QLineEdit {background: rgb(255, 102, 74);}")
+        else:
+            self._ui.le_name.setStyleSheet(self._default_color)
 
     # ----------------------------------------------------------------------
     def accept(self):
@@ -93,6 +143,7 @@ class ConfigureDevice(QtWidgets.QDialog):
                                                              QtWidgets.QLineEdit.Normal, "")
             if not okPressed:
                 self.reject()
+                return
 
         if self.new_device:
             self._make_new_device(name)
@@ -101,23 +152,31 @@ class ConfigureDevice(QtWidgets.QDialog):
 
         super(ConfigureDevice, self).accept()
 
+    @staticmethod
+    # ----------------------------------------------------------------------
+    def _fill_with_sub_elements(device, container):
+        for widget in container.values():
+            valid, name, value = widget.get_data()
+            if valid:
+                property_element = ET.SubElement(device, name)
+                property_element.text = value
+
     # ----------------------------------------------------------------------
     def _make_new_device(self, name):
-        type = self._ui.cmb_type.currentText()
-        self.new_device = ET.Element(type, {'name': name, 'active': 'no', 'comment': self._ui.le_comment.text()})
+        self.new_device = ET.Element(self._type, {'name': name, 'active': 'no', 'comment': self._ui.le_comment.text()})
 
-        if type in ['serial_device', 'single_device']:
+        if self._type in ['serial_device', 'single_device']:
+
+            if self._type == 'serial_device':
+                self._fill_with_sub_elements(self.new_device, self._common_property_widgets)
+                device = ET.SubElement(self.new_device, 'single_device', attrib={'name': self._ui.le_name.text(),
+                                                                                 'active': 'yes',
+                                                                                 'comment': self._ui.le_comment.text()})
+
+                self._fill_with_sub_elements(device, self._personal_property_widgets)
 
-            if type == 'serial_device':
-                container = self._common_property_widgets
             else:
-                container = self._personal_property_widgets
-
-            for widget in container.values():
-                valid, name, value = widget.get_data()
-                if valid:
-                    property_element = ET.SubElement(self.new_device, name)
-                    property_element.text = value
+                self._fill_with_sub_elements(self.new_device, self._personal_property_widgets)
 
     # ----------------------------------------------------------------------
     def _modify_device(self):
@@ -149,35 +208,30 @@ class ConfigureDevice(QtWidgets.QDialog):
         super(ConfigureDevice, self).accept()
 
     # ----------------------------------------------------------------------
-    def _change_device_type(self, text):
+    def _apply_template(self):
         self._common_property_widgets = {}
-        if text in ['group', 'configuration']:
-            self._ui.fr_common_properties.setVisible(False)
-            self._ui.fr_personal_properties.setVisible(False)
-        elif text == 'serial_device':
-            self._ui.fr_common_properties.setVisible(True)
-            self._ui.fr_personal_properties.setVisible(False)
-        else:
-            self._ui.fr_common_properties.setVisible(False)
-            self._ui.fr_personal_properties.setVisible(True)
+        self._personal_property_widgets = {}
+
+        template = self._ui.cmb_template.currentText()
+        for device in self.templates.iter('device'):
+            if device.get('name') == template:
+                for child in list(device):
+                    for key, value in child.attrib.items():
+                        if value == '___local___':
+                            value = PyTango.Database().get_db_host().split('.')[0] + ":10000"
+                        if self._type == 'single_device':
+                            self._add_property('personal', key, value)
+                        else:
+                            self._add_property(child.tag, key, value)
 
         self._bild_view()
 
     # ----------------------------------------------------------------------
-    def _add_property(self, type, name='', value='', doRefresh=True):
-
-        if self._type == 'serial' and type == 'personal':
-            if name == '':
-                name = 'sardananame'
-            if name in ['device', 'sardananame']:
-                new_property = PropertyWidget(self, self._last_ind, name, value, True)
-            else:
-                return
-        else:
-            new_property = PropertyWidget(self, self._last_ind, name, value)
+    def _add_property(self, property_type, name='', value='', editable=True, doRefresh=True):
 
+        new_property = PropertyWidget(self, self._last_ind, name, value, editable)
         new_property.delete_me.connect(self._delete_property)
-        if type == 'common':
+        if property_type == 'common':
             self._common_property_widgets[self._last_ind] = new_property
         else:
             self._personal_property_widgets[self._last_ind] = new_property
